package joot.tree;

import java.util.stream.Stream;

import org.jooq.Field;
import org.jooq.Record;
import org.jooq.RecordType;
import org.jooq.Row;
import org.jooq.RowN;
import org.jooq.Table;
import org.jooq.TableField;
import org.jooq.TableRecord;
import org.jooq.impl.DSL;

/**
 * A type-safe key implementation.
 */
public class TableKey<R extends TableRecord<R>, K extends Record> {

	private final Table<R> table;
	private final TableField<R, ?>[] fields;
	
	// TODO Find something better than casting RecordType fields to TableField.
	//      Or at least handle exception better.
	@SuppressWarnings("unchecked")
	public TableKey(Table<R> table, RecordType<K> type) {
		this.table = table;
		this.fields = Stream.of(type.fields()).map(TableField.class::cast).toArray(TableField[]::new);
	}
	
	public Table<R> getTable() {
		return table;
	}
	
	public TableField<R, ?>[] getFields() {
		return fields;
	}
	
	@Override
	public String toString() {
		return fieldsRow().toString();
	}
	
//////// HELPERS
	
	public RowN fieldsRow() {
		return DSL.row(fields);
	}
	
	public RowN valuesRow(Record record) {
		// Not using Record.valuesRow() so we get RowN rather than Row
		return DSL.row(record.into(fields).intoArray());
	}
	
	@SuppressWarnings("unchecked")
	public K extract(R record) {
		return (K) record.into(fields);
	}
	
//////// HELPERS FOR RELATION HANDLING
	
	@SuppressWarnings("unchecked")
	public <O extends TableRecord<O>> void copyValues(R from, O to, TableKey<O, K> toKey) {
		if (!matches(toKey))
			throw new IllegalArgumentException(this + " != " + toKey);
		for (int i = 0; i < fields.length; i++)
			to.set((Field<Object>) toKey.fields[i], (Object) from.get(fields[i]));	
	}
	
	public void nullifyValues(R record) {
		for (Field<?> f : fields) record.set(f, null);
	}
	
//////// CHECKS
	
	public boolean matches(TableKey<?, ?> other) {
		if (fields.length != other.fields.length)
			return false;
		for (int i = 0; i < fields.length; i++)
			if (!fields[i].getDataType().equals(other.fields[i].getDataType()))
				return false;
		return true;
	}
	
	public boolean matches(Row other) {
		if (fields.length != other.size())
			return false;
		for (int i = 0; i < fields.length; i++)
			if (!fields[i].getDataType().equals(other.field(i).getDataType()))
				return false;
		return true;
	}

}
