package joot.tree.action;

import org.jooq.DSLContext;
import org.jooq.Record;
import org.jooq.TableRecord;

import joot.tree.TreeNode;
import joot.tree.TreeLink.OneToMany;
import joot.tree.TreeLink.OneToOne;

/**
 * Abstract super-class for INSERT/UPDATE/MERGE actions.
 */
public abstract class AbstractWriteAction<KR extends Record, PR extends TableRecord<PR>> {

	protected final TreeNode<KR, PR> node;
	protected final DSLContext dsl;
	
	protected PR parent;
	
	public AbstractWriteAction(TreeNode<KR, PR> node, DSLContext dsl) {
		this.node = node;
		this.dsl = dsl;
	}
	
	public int execute(PR parent) {
		this.parent = parent;
		int count = 0;
		count += parent();
		count += node.getOneToOneLinks().mapToInt(this::child).sum();
		count += node.getOneToManyLinks().mapToInt(this::child).sum();
		return count;
	}
	
	protected abstract int parent();
	protected abstract <CR extends TableRecord<CR>> int child(OneToOne<KR, PR, CR> link);
	protected abstract <CR extends TableRecord<CR>> int child(OneToMany<KR, PR, CR> link);

}
