package joot.tree.action;

import org.jooq.DSLContext;
import org.jooq.Row;
import org.jooq.RowN;
import org.jooq.TableRecord;
import org.jooq.impl.DSL;

import joot.tree.TreeNode;

public class DeleteAction<PR extends TableRecord<PR>> {

	private final TreeNode<?, PR> node;
	private final DSLContext dsl;
	
	public DeleteAction(TreeNode<?, PR> node, DSLContext dsl) {
		this.node = node;
		this.dsl = dsl;
	}
	
	public int execute(Row parentKey) {

		if (!node.matches(parentKey))
			throw new IllegalArgumentException("Given key values doesn't match node key type");
		
		RowN key = DSL.row(parentKey.fields());
		int count = 0;
		
		count += node
				.getLinks()
				.mapToInt(link -> {
					return dsl
						.deleteFrom(link.getTable())
						.where(link.fieldsRow().eq(key))
						.execute();
				}).sum();
		
		count += dsl
				.deleteFrom(node.getTable())
				.where(node.fieldsRow().eq(key))
				.execute();
		
		return count;
	}
	
}
