package joot.tree.action;

import org.jooq.DSLContext;
import org.jooq.QueryPart;
import org.jooq.RenderContext;
import org.jooq.TableRecord;
import org.jooq.conf.ParamType;
import org.jooq.conf.RenderNameStyle;
import org.jooq.impl.DSL;
import org.jooq.impl.DefaultConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import joot.tree.TreeNode;

public class DumpAction<PR extends TableRecord<PR>> {

	private final static Logger logger = LoggerFactory.getLogger(DumpAction.class);
	
	private final TreeNode<?, PR> tree;
	private final RenderContext ctx;
	
	public DumpAction(TreeNode<?, PR> tree) {
		DSLContext dsl = DSL.using(new DefaultConfiguration());
		dsl.configuration().settings().withRenderNameStyle(RenderNameStyle.AS_IS);
		this.ctx = dsl.renderContext().qualify(false).paramType(ParamType.INLINED);
		this.tree = tree;
	}
	
	private void debug(String fmt, QueryPart... parts) {
		if (logger.isDebugEnabled())
			logger.debug(ctx.render(DSL.sql(fmt, parts)));
	}
	
	public void dump() {
		dump(null);
	}
	
	// TODO Use DSLContext.newResult().format() to dump records 
	public void dump(PR record) {
		
		debug("ONE     : {0}", tree.getTable());
		if (record != null)
			logger.debug("\t{}", record.intoMap());
		
		tree.getOneToOneLinks().forEach(link -> {
			debug("TO-ONE  : {0}{1} to {2}{3}", tree.getTable(), tree.fieldsRow(), link.getTable(), link.fieldsRow());
			if (record != null)
				link.optional(record).ifPresent(cr -> logger.debug("\t{}", link.get(record).intoMap()));
		});
		
		tree.getOneToManyLinks().forEach(link -> {
			debug("TO-MANY : {0}{1} to {2}{3}", tree.getTable(), tree.fieldsRow(), link.getTable(), link.fieldsRow());
			if (record != null)
				link.stream(record).forEach(cr -> logger.debug("\t{}", cr.intoMap()));
		});
	}

}
