package joot.tree.action;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import org.jooq.Condition;
import org.jooq.DSLContext;
import org.jooq.RowN;
import org.jooq.TableRecord;

import joot.tree.TableKey;
import joot.tree.TreeNode;
import joot.tree.TreeLink.OneToMany;
import joot.tree.TreeLink.OneToOne;

public class FetchAction<PR extends TableRecord<PR>> {

	private final TreeNode<?, PR> tree;
	private final DSLContext dsl;
	
	private Condition condition;
	private Map<RowN, PR> parents;

	public FetchAction(TreeNode<?, PR> tree, DSLContext dsl) {
		this.tree = tree;
		this.dsl = dsl;
	}
	
	public Collection<PR> fetch(Condition condition) {
		this.condition = condition;
		parent(tree);
		tree.getOneToOneLinks().forEach(this::child);
		tree.getOneToManyLinks().forEach(this::child);
		return parents.values();
	}

	private void parent(TableKey<PR, ?> key) {
		parents = dsl
			.selectFrom(key.getTable())
			.where(condition)
			.fetch()
			.intoMap(key::valuesRow);
	}
	
	private <CR extends TableRecord<CR>> void child(OneToOne<?, PR, CR> link) {
		dsl
			.selectFrom(link.getTable())
			.where(link.fieldsRow().in(parents.keySet()))
			.fetch()
			.stream()
			.forEach(cr -> {
				PR pr = parents.get(link.valuesRow(cr));
				link.set(pr, cr);
				link.nullifyValues(cr);
			});
	}
	
	private <CR extends TableRecord<CR>> void child(OneToMany<?, PR, CR> link) {
		dsl
			.selectFrom(link.getTable())
			.where(link.fieldsRow().in(parents.keySet()))
			.fetch()
			.stream()
			.forEach(cr -> {
				PR pr = parents.get(link.valuesRow(cr));
				link.add(pr, cr, ArrayList::new);
				link.nullifyValues(cr);
			});
	}
	
}
