package joot.tree.action;

import static joot.tree.util.JooqUtils.attach;

import org.jooq.DSLContext;
import org.jooq.Record;
import org.jooq.TableRecord;

import joot.tree.TreeLink;
import joot.tree.TreeNode;
import joot.tree.TreeLink.OneToMany;
import joot.tree.TreeLink.OneToOne;

public class InsertAction<KR extends Record, PR extends TableRecord<PR>> extends AbstractWriteAction<KR, PR> {
	
	public InsertAction(TreeNode<KR, PR> tree, DSLContext dsl) {
		super(tree, dsl);
	}
	
	@Override
	protected int parent() {
		return attach(parent, dsl).insert();
	}
	
	@Override
	protected <CR extends TableRecord<CR>> int child(OneToOne<KR, PR, CR> link) {
		CR child = link.get(parent);
		return child != null ? insertChild(child, link) : 0;
	}
	
	@Override
	protected <CR extends TableRecord<CR>> int child(OneToMany<KR, PR, CR> link) {
		return link
			.stream(parent)
			.mapToInt(child -> insertChild(child, link))
			.sum();
	}
	
	private <CR extends TableRecord<CR>> int insertChild(CR child, TreeLink<KR, PR, CR> link) {
		node.copyValues(parent, child, link);
		return attach(child, dsl).insert();
	}

}
