package joot.tree.node;

import java.util.List;
import java.util.function.BiConsumer;
import java.util.function.Function;

import org.jooq.Record1;
import org.jooq.Table;
import org.jooq.TableField;
import org.jooq.UpdatableRecord;
import org.jooq.impl.DSL;

import joot.tree.TreeNode;
import joot.tree.TreeLink.OneToMany;
import joot.tree.TreeLink.OneToOne;

/**
 * Basic TreeNode implementation with single-column key.
 *
 * @param <ID> Key type
 * @param <PR> Parent table type
 */
public class SimpleNode<ID, PR extends UpdatableRecord<PR>> extends TreeNode<Record1<ID>, PR> {
	
	public SimpleNode(Table<PR> table, TableField<PR, ID> id) {
		super(table, DSL.recordType(id));
	}
	
	public static <ID, PR extends UpdatableRecord<PR>> SimpleNode<ID, PR> one(Table<PR> table, TableField<PR, ID> id) {
		return new SimpleNode<>(table, id);
	}
	
	public <CR extends UpdatableRecord<CR>> SimpleNode<ID, PR> toOne(Table<CR> table, TableField<CR, ID> id, Function<PR, CR> getter, BiConsumer<PR, CR> setter) {
		link(new OneToOne<>(
			table, DSL.recordType(id), getter, setter
		));
		return this;
	}
	
	public <CR extends UpdatableRecord<CR>> SimpleNode<ID, PR> toMany(Table<CR> table, TableField<CR, ID> id, Function<PR, List<CR>> getter, BiConsumer<PR, List<CR>> setter) {
		link(new OneToMany<>(
			table, DSL.recordType(id), getter, setter
		));
		return this;
	}
	
}
