package joot.tree.util;

import org.jooq.Attachable;
import org.jooq.DSLContext;
import org.jooq.Field;
import org.jooq.Record;
import org.jooq.TableRecord;

public abstract class JooqUtils {

	/**
	 * Returns TRUE if all primary key fields are not NULL.
	 */
	public static boolean isPrimaryKeyNotNull(TableRecord<?> rec) {
		for (Field<?> f : rec.getTable().getPrimaryKey().getFieldsArray())
			if (rec.getValue(f) == null)
				return false;
		return true;
	}
	
	/**
	 * Copy this record into a new record holding only the fields being part of the primary key.
	 */
	public static Record intoPrimaryKey(TableRecord<?> rec) {
		return rec.into(rec.getTable().getPrimaryKey().getFieldsArray());
	}
	
	/**
	 * Attach a configuration to an attachable and return it.
	 */
	// TODO Handle detach too (using a lambda as argument)
	public static <T extends Attachable> T attach(T t, DSLContext dsl) {
		t.attach(dsl.configuration());
		return t;
	}
}